<!doctype html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">    
    <title>PowerfulSeal: Chaos Engineering</title>

    <link rel="stylesheet" href="./static/bootstrap/4.4.1/css/bootstrap.min.css" integrity="sha384-Vkoo8x4CGsO3+Hhxv8T/Q5PaXtkKtu6ug5TOeNV6gBiFeWPGFN9MuhOf23Q9Ifjh" crossorigin="anonymous">

    <style>
        [role="main"] {
            padding-top: 135px;
        }
        @media (min-width: 768px) {
            [role="main"] {
                padding-top: 50px;
            }
        }
    </style>
</head>
<body>

<nav class="navbar navbar-dark fixed-top bg-dark flex-md-nowrap p-0 shadow">
    <a class="navbar-brand col-sm-3 col-md-2 mr-0" href="#">PowerfulSeal</a>
</nav>

<div class="container-fluid">
  <div class="row">
    <nav role="main" class="col-md-2 d-none d-md-block bg-light sidebar">
      <div class="sidebar-sticky">
        <ul class="nav flex-column">
          <li class="nav-item">
            <a class="nav-link active" href="{{ url_for('index') }}">
              Policy
            </a>
          </li>
          <li class="nav-item">
            <a class="nav-link" href="{{ url_for('logs') }}">
              Logs
            </a>
          </li>
          <li class="nav-item">
            <a class="nav-link" href="{{ url_for('static', filename='schema/index.html') }}">
              Schema
            </a>
          </li>
        </ul>
      </div>
    </nav>

    <main role="main" class="col-md-9 ml-sm-auto col-lg-10 px-4">

      <div class="d-flex justify-content-between flex-wrap flex-md-nowrap align-items-center pt-3 pb-2 mb-3 border-bottom">
        <h1 class="h2">{% block title %}{% endblock %}</h1>
      </div>

{% block content %}{% endblock %}

    </main>
  </div>
</div>


</body>
</html>
